#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_sequence.interfaces import ISequentialIntIds, ISequentialIdInfo, IInternalReference, IInternalReferencesList
from pyams_utils.interfaces.site import ISiteGenerations

# import packages
from pyams_catalog.index import FieldIndexWithInterface, KeywordIndexWithInterface
from pyams_catalog.site import check_required_indexes
from pyams_sequence.utility import SequentialIntIds
from pyams_utils.registry import utility_config
from pyams_utils.site import check_required_utilities


REQUIRED_UTILITIES = ((ISequentialIntIds, '', SequentialIntIds, 'Sequential IDs'), )


REQUIRED_INDEXES = [('oid', FieldIndexWithInterface, {'interface': ISequentialIdInfo,
                                                      'discriminator': 'hex_oid'}),
                    ('link_reference', FieldIndexWithInterface, {'interface': IInternalReference,
                                                                 'discriminator': 'reference'}),
                    ('link_references', KeywordIndexWithInterface, {'interface': IInternalReferencesList,
                                                                    'discriminator': 'references'})]


@utility_config(name='PyAMS sequence', provides=ISiteGenerations)
class SequenceGenerationsChecker(object):
    """PyAMS sequence generations checker"""

    order = 60
    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)
        check_required_indexes(site, REQUIRED_INDEXES)
